@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2012 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ module Img = Image.RGBA32
 
 class effect ~kind effect (source:source) =
 object (self)
-  inherit operator kind [source] as super
+  inherit operator ~name:"video.effect" kind [source] as super
 
   method stype = source#stype
 
