@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2012 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -446,44 +446,34 @@ class output ~kind p =
                                                                     ~name:
                                                                     mount
                                                                     source
-                                                                  
                                                                 (** File descriptor where to dump. *)
                                                                 val mutable
                                                                   dump = None
-                                                                  
                                                                 val mutable
                                                                   encoder =
                                                                   None
-                                                                  
                                                                 val mutable
                                                                   clients =
                                                                   Queue.
                                                                     create ()
-                                                                  
                                                                 val clients_m =
                                                                   Mutex.
                                                                     create ()
-                                                                  
                                                                 val duppy_c =
                                                                   Duppy_c.
                                                                     create ()
-                                                                  
                                                                 val duppy_m =
                                                                   Duppy_m.
                                                                     create ()
-                                                                  
                                                                 val mutable
                                                                   chunk_len =
                                                                   0
-                                                                  
                                                                 val mutable
                                                                   burst_data =
                                                                   []
-                                                                  
                                                                 val mutable
                                                                   burst_pos =
                                                                   0
-                                                                  
                                                                 val metadata =
                                                                   {
                                                                     metadata =
@@ -492,7 +482,6 @@ class output ~kind p =
                                                                     Mutex.
                                                                     create ();
                                                                   }
-                                                                  
                                                                 method encode =
                                                                   fun frame
                                                                     ofs len
@@ -504,7 +493,6 @@ class output ~kind p =
                                                                     encode
                                                                     frame ofs
                                                                     len
-                                                                  
                                                                 method insert_metadata =
                                                                   fun m ->
                                                                     let m 
@@ -557,7 +545,6 @@ class output ~kind p =
                                                                     Meta.
                                                                     export_metadata
                                                                     meta))
-                                                                  
                                                                 method add_client =
                                                                   fun
                                                                     ~protocol
@@ -859,7 +846,6 @@ class output ~kind p =
                                                                     Harbor.
                                                                     relayed
                                                                     reply)))
-                                                                  
                                                                 method send =
                                                                   fun b ->
                                                                     let slen 
@@ -1061,7 +1047,6 @@ class output ~kind p =
                                                                     new_clients))
                                                                     ()))))
                                                                     else ()
-                                                                  
                                                                 method output_start =
                                                                   (assert
                                                                     (encoder
@@ -1079,25 +1064,12 @@ class output ~kind p =
                                                                     Meta.
                                                                     empty_metadata);
                                                                     let handler 
-                                                                    ~http_method
                                                                     ~protocol
                                                                     ~data
                                                                     ~headers
                                                                     ~socket
                                                                     uri =
-                                                                    if
-                                                                    http_method
-                                                                    <> "GET"
-                                                                    then
-                                                                    Harbor.
-                                                                    reply
-                                                                    (Harbor.
-                                                                    http_error_page
-                                                                    405
-                                                                    "Method Not Allowed"
-                                                                    "Method not allowed!")
-                                                                    else
-                                                                    (let rex 
+                                                                    let rex 
                                                                     =
                                                                     Pcre.
                                                                     regexp
@@ -1137,12 +1109,13 @@ class output ~kind p =
                                                                     ~headers
                                                                     ~uri
                                                                     ~args
-                                                                    socket)
+                                                                    socket
                                                                     in
                                                                     (Harbor.
                                                                     add_http_handler
                                                                     ~port
-                                                                    ~uri
+                                                                    ~verb:
+                                                                    `Get ~uri
                                                                     handler;
                                                                     match dumpfile
                                                                     with
@@ -1155,7 +1128,6 @@ class output ~kind p =
                                                                     | 
                                                                     None ->
                                                                     ())))
-                                                                  
                                                                 method output_stop =
                                                                   (ignore
                                                                     ((Utils.
@@ -1168,7 +1140,9 @@ class output ~kind p =
                                                                    Harbor.
                                                                     remove_http_handler
                                                                     ~port
-                                                                    ~uri ();
+                                                                    ~verb:
+                                                                    `Get ~uri
+                                                                    ();
                                                                    let new_clients 
                                                                     =
                                                                     Queue.
@@ -1215,13 +1189,11 @@ class output ~kind p =
                                                                     | 
                                                                     None ->
                                                                     ()))
-                                                                  
                                                                 method output_reset =
                                                                   (self#
                                                                     output_stop;
                                                                    self#
                                                                     output_start)
-                                                                  
                                                               end
   
 let () =
