@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2012 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,29 +28,26 @@ open Dtools
 
 let conf =
   Conf.void ~p:(Configure.conf#plug "request") "requests configuration"
-let conf_max_rid =
-  Conf.int ~p:(conf#plug "max_id") ~d:Configure.requests_max_id
-    "maximum number of requests"
+let grace_time =
+  Conf.float ~p:(conf#plug "grace_time") ~d:600.
+    "Time after which a destroyed request cannot be accessed anymore."
 
 let log = Log.make ["request"]
 
 (** File utilities. *)
 
-let remove_file_proto =
-  (* TODO Use PCRE *)
-  let re = Str.regexp "^file://" in
-    fun s ->
-      Str.replace_first re "" s
+let remove_file_proto s =
+  Pcre.substitute ~pat:"^file://" ~subst:(fun _ -> "") s
 
 let home_unrelate s = Utils.home_unrelate (remove_file_proto s)
 
 let parse_uri uri =
   try
     let i = String.index uri ':' in
-      (String.sub uri 0 i),
-      (String.sub uri (i+1) ((String.length uri)-(i+1)))
+      Some ((String.sub uri 0 i),
+            (String.sub uri (i+1) ((String.length uri)-(i+1))))
   with
-    | _ -> "",""
+    | _ -> None
 
 let cleanup =
   let re1 = Str.regexp "^[\t ]*" in
@@ -70,60 +67,17 @@ let get_extension f =
 
 type metadata = (string,string) Hashtbl.t
 
-(* These two functions are taken from Extlib's module UTF8
- * Copyright (c) 2002, 2003 Yamagata Yoriyuki *)
-let rec utf8_search_head s i =
-  if i >= String.length s then i else
-  let n = Char.code (String.unsafe_get s i) in
-  if n < 0x80 || n >= 0xc2 then i else
-  utf8_search_head s (i + 1)
-let utf8_next s i =
-  let n = Char.code s.[i] in
-  if n < 0x80 then i + 1 else
-  if n < 0xc0 then utf8_search_head s (i + 1) else
-  if n <= 0xdf then i + 2
-  else if n <= 0xef then i + 3
-  else if n <= 0xf7 then i + 4
-  else if n <= 0xfb then i + 5
-  else if n <= 0xfd then i + 6
-  else failwith "Request.utf_8.next"
-(* End of Extlib code *)
-
-(* The utf8_S_formatter provides some of the features of the standard S
- * formatter, but supports UTF8 encoded strings. So it won't escape wide
- * characters and mistake continuations of characters with normal ones.
- * Currently, it escapes '\n' and '"'. *)
-let utf8_S_formatter f s =
-  let i = ref 0 in
-  let next = utf8_next s in
-  let l = String.length s in
-  let out = Format.pp_print_char f in
-  let outs = Format.pp_print_string f in
-  let out_sub i j =
-    for n = i to j do out s.[n] done
-  in
-    out '"' ;
-    while !i < l do
-      let n = next !i in
-      if n-1 < l then
-        if n > !i+1 then out_sub !i (n-1) else
-          if s.[!i] = '"' then outs "\\\"" else
-          if s.[!i] = '\n' then outs "\\n" else
-            out s.[!i] ;
-      i := n
-    done ;
-    out '"'
-
 let string_of_metadata metadata =
   let b = Buffer.create 20 in
   let f = Format.formatter_of_buffer b in
+  let escape f s = Utils.escape_utf8 f s in
   let first = ref true in
     Hashtbl.iter (fun k v ->
                     if !first then begin
                       first := false ;
-                      Format.fprintf f "%s=%a" k utf8_S_formatter v
+                      Format.fprintf f "%s=%a" k escape v
                     end else
-                      Format.fprintf f "\n%s=%a" k utf8_S_formatter v)
+                      Format.fprintf f "\n%s=%a" k escape v)
       metadata ;
     Format.pp_print_flush f () ;
     Buffer.contents b
@@ -156,7 +110,7 @@ let string_of_log log =
                      Printf.sprintf "\n[%s] %s" (pretty_date date) msg)) "" log
 
 (** Requests.
-  * The purpose of a request is to get a valid file. The file can be audio,
+  * The purpose of a request is to get a valid file. The file can contain media
   * in which case validity implies finding a working decoder, or can be
   * something arbitrary, like a playlist.
   * This file is fetched using protocols. For example the fetching can involve
@@ -189,14 +143,10 @@ type indicator = {
 
 type status = Idle | Resolving | Ready | Playing | Destroyed
 
-type audio = [ `Audio ]
-type raw   = [ `Raw ]
-type flags = [ audio | raw ]
-
-type _t = {
+type t = {
   id : int ;
   initial_uri : string ;
-  audio : bool ;
+  kind : Frame.content_kind option ; (* No kind for raw requests *)
   persistent : bool ;
 
   (* The status of a request gives partial information of what's being done
@@ -214,12 +164,10 @@ type _t = {
   log : log ;
   mutable root_metadata : metadata ;
   mutable indicators : indicator list list ;
-  mutable decoder : (unit -> Decoder.decoder) option ;
+  mutable decoder : (unit -> Decoder.file_decoder) option ;
 }
-type +'a t = _t constraint 'a = [<flags]
 
-let to_raw   x = x
-let to_audio x = assert x.audio ; x
+let kind x = x.kind
 
 let indicator ?(metadata=Hashtbl.create 10) ?temporary s = {
   string = home_unrelate s ;
@@ -301,41 +249,94 @@ let rec pop_indicator t =
         Unix.unlink i.string
       with
         | e ->
-            log#f 2 "Unlink failed: %S" (Printexc.to_string e)
+            log#f 2 "Unlink failed: %S" (Utils.error_message e)
       end ;
     t.decoder <- None ;
     if repop then pop_indicator t
 
+let conf_metadata_decoders =
+  Dtools.Conf.list
+    ~p:(conf#plug "metadata_decoders") ~d:[]
+    "Decoders and order used to decode files' metadata."
+
+let f c v =
+  match c#get_d with
+    | None -> c#set_d (Some [v])
+    | Some d -> c#set_d (Some (d@[v]))
+
+let get_decoders conf decoders =
+  let f cur name =
+    match decoders#get name with
+      | Some p -> (name,p)::cur
+      | None   -> log#f 2 "Cannot find decoder %s" name;
+                  cur
+  in
+  List.fold_left f [] (List.rev conf#get)
+
 let mresolvers_doc =
   "Methods to extract metadata from a file."
 let mresolvers =
-  Plug.create ~doc:mresolvers_doc ~insensitive:true "metadata formats"
+  Plug.create 
+    ~register_hook:(fun (name,_) -> f conf_metadata_decoders name)
+    ~doc:mresolvers_doc ~insensitive:true "metadata formats"
+
+let conf_override_metadata =
+  Conf.bool ~p:(conf_metadata_decoders#plug "override") ~d:false
+      "Allow metadata resolvers to override metadata already \
+       set through annotate: or playlist resolution for instance."
+
+(** Sys.file_exists doesn't make a difference between existing files
+  * and files without enough permissions to list their attributes,
+  * for example when they are in a directory without x permission.
+  * The two following functions allow a more precise diagnostic.
+  * We do not use them everywhere in this file, but only when splitting
+  * existence and readability checks yields better logs. *)
+
+let file_exists name =
+  try Unix.access name [Unix.F_OK] ; true with
+    | Unix.Unix_error (Unix.EACCES,_,_) -> true
+    | Unix.Unix_error _ -> false
+
+let file_is_readable name =
+  try Unix.access name [Unix.R_OK] ; true with
+    | Unix.Unix_error _ -> false
 
 let local_check t =
-  if t.audio then try
-    while t.decoder = None && Sys.file_exists (peek_indicator t).string do
+  let check_decodable kind = try
+    while t.decoder = None && file_exists (peek_indicator t).string do
       let indicator = peek_indicator t in
       let name = indicator.string in
-        match Decoder.search_valid name with
-          | Some (format,f) ->
+      let metadata = get_all_metadata t in
+        if not (file_is_readable name) then begin
+          log#f 3 "Read permission denied for %S!" name ;
+          add_log t "Read permission denied!" ;
+          pop_indicator t
+        end else
+        match Decoder.get_file_decoder ~metadata name kind with
+          | Some (decoder_name,f) ->
               t.decoder <- Some f ;
-              mresolvers#iter
-                (fun _ resolver ->
+              set_root_metadata t "decoder" decoder_name ;
+              List.iter
+                (fun (_,resolver) ->
                    try
-                     let ans = resolver ~format name in
+                     let ans = resolver name in
                        List.iter
                          (fun (k,v) ->
-                            (* XXX Policy ? Never/always overwrite ? *)
-                            Hashtbl.replace indicator.metadata
-                              (String.lowercase k) (cleanup v))
+                           let k = String.lowercase k in
+                           if conf_override_metadata#get || get_metadata t k = None then
+                             Hashtbl.replace indicator.metadata
+                              k (cleanup v))
                          ans;
                    with
-                     | _ -> ()) ;
+                     | _ -> ()) (get_decoders conf_metadata_decoders
+                                     mresolvers) ;
               t.status <- Ready
           | None -> pop_indicator t
       done
   with
     | No_indicator -> ()
+  in
+    match t.kind with None -> () | Some k -> check_decodable k
 
 let push_indicators t l =
   if l <> [] then
@@ -353,7 +354,7 @@ let push_indicators t l =
 let is_ready t =
   t.indicators <> [] &&
   Sys.file_exists (peek_indicator t).string &&
-  ( t.decoder <> None || not t.audio )
+  ( t.decoder <> None || t.kind = None )
 
 (** [get_filename request] returns
   * [Some f] if the request successfully lead to a local file [f],
@@ -375,7 +376,7 @@ exception Duration of float
 let duration file =
   try
     dresolvers#iter
-      (fun format resolver ->
+      (fun name resolver ->
         try
          let ans = resolver file in
          raise (Duration ans)
@@ -411,6 +412,11 @@ let update_metadata t =
           replace "on_air" (pretty_date (Unix.localtime d))
       | None -> ()
     end ;
+    begin match t.kind with
+      | None -> ()
+      | Some k ->
+          replace "kind" (Frame.string_of_content_kind k)
+    end ;
     replace "status"
       (match t.status with
          | Idle -> "idle"
@@ -423,119 +429,66 @@ let get_metadata t k = update_metadata t ; get_metadata t k
 let get_all_metadata t = update_metadata t ; get_all_metadata t
 let get_root_metadata t = update_metadata t ; get_root_metadata t
 
-(** Request global management.
-  Access to the global table is thread-safe, unlike access to a request.
-  In some cases (trace) there could be a problem : TODO ? *)
+(** Global management *)
 
-let requests = Hashtbl.create Configure.requests_table_size
-(* Maximum number of alive requests *)
-
-let lock = Mutex.create ()
-
-exception Found of int
-
-let next =
-  let current = ref (-1) in fun () ->
-    let request_max = conf_max_rid#get in
-    assert (not (Mutex.try_lock lock)) ;
-    try
-      for i = 1 to request_max do
-        current := (!current+1) mod request_max ;
-        begin try
-          let t = Hashtbl.find requests !current in
-            if t.status = Destroyed then
-              raise (Found !current)
-        with
-          | Not_found -> raise (Found !current)
-        end
-      done ;
-      (-1)
-    with
-      | Found r -> r
-
-(** Query request state and get requests by state *)
+module Pool =
+  Pool.Make(struct type req = t type t = req end)
 
 let get_id t = t.id
 
-let from_id id =
-  Mutex.lock lock ;
-  try
-    let r = Hashtbl.find requests id in
-      Mutex.unlock lock ;
-      Some r
-  with
-    | Not_found -> Mutex.unlock lock ; None
+let from_id id = Pool.find id
+
+let all_requests () =
+  Pool.fold (fun k v l -> k::l) []
 
 let alive_requests () =
-  Mutex.lock lock ;
-  let l =
-    Hashtbl.fold
-      (fun k v l -> if v.status <> Destroyed then k::l else l)
-      requests []
-  in
-    Mutex.unlock lock ;
-    l
+  Pool.fold
+    (fun k v l -> if v.status <> Destroyed then k::l else l)
+    []
 
 let is_on_air t = t.on_air <> None
 
 let on_air_requests () =
-  Mutex.lock lock ;
-  let l =
-    Hashtbl.fold
-      (fun k v l -> if is_on_air v then k::l else l)
-      requests []
-  in
-    Mutex.unlock lock ;
-    l
+  Pool.fold
+    (fun k v l -> if is_on_air v then k::l else l)
+    []
 
 let is_resolving t = t.status = Resolving
 
 let resolving_requests () =
-  Mutex.lock lock ;
-  let l =
-    Hashtbl.fold
-      (fun k v l -> if is_resolving v then k::l else l)
-      requests []
-  in
-    Mutex.unlock lock ;
-    l
+  Pool.fold
+    (fun k v l -> if is_resolving v then k::l else l)
+    []
 
 (** Creation *)
 
-let _create ~audio ?(metadata=[]) ?(persistent=false) ?(indicators=[]) u =
-  Mutex.lock lock ;
-  let rid = next () in
-    if rid = -1 then
-      ( Mutex.unlock lock ;
-        log#f 2 "No available RID!" ;
-        None )
-    else
-      let t = {
-        id = rid ;
-        initial_uri = u ;
-        audio = audio ;
-        persistent = persistent ;
+let create ~kind ?(metadata=[]) ?(persistent=false) ?(indicators=[]) u =
+  let rid,register = Pool.add () in
+  let t = {
+    id = rid ;
+    initial_uri = u ;
+    kind = kind ;
+    persistent = persistent ;
 
-        on_air = None ;
-        resolving = None ;
-        status = Idle ;
+    on_air = None ;
+    resolving = None ;
+    status = Idle ;
 
-        decoder = None ;
-        log = Queue.create () ;
-        root_metadata = Hashtbl.create 10 ;
-        indicators = [] }
-      in
-        Hashtbl.replace requests t.id t ;
-        Mutex.unlock lock ;
-        List.iter
-          (fun (k,v) -> Hashtbl.replace t.root_metadata k v)
-          metadata ;
-        push_indicators t
-          (if indicators=[] then [indicator u] else indicators) ;
-        Some t
+    decoder = None ;
+    log = Queue.create () ;
+    root_metadata = Hashtbl.create 10 ;
+    indicators = [] }
+  in
+    register t ;
+    List.iter
+      (fun (k,v) -> Hashtbl.replace t.root_metadata k v)
+      metadata ;
+    push_indicators t
+      (if indicators=[] then [indicator u] else indicators) ;
+    t
 
-let create = _create ~audio:true
-let create_raw = _create ~audio:false
+let create_raw = create ~kind:None
+let create ~kind = create ~kind:(Some kind)
 
 let on_air t =
   t.on_air <- Some (Unix.time ()) ;
@@ -554,12 +507,12 @@ let destroy ?force t =
     while t.indicators <> [] do pop_indicator t done ;
     t.status <- Destroyed ;
     add_log t "Request finished." ;
+    Pool.kill t.id grace_time#get
   end
 
 let clean () =
-  Hashtbl.iter
+  Pool.iter
     (fun k r -> if r.status <> Destroyed then destroy ~force:true r)
-    requests
 
 let get_decoder r =
   match r.decoder with None -> None | Some d -> Some (d ())
@@ -579,10 +532,13 @@ let is_static s =
   if Sys.file_exists (home_unrelate s) then
     Some true
   else
-    let proto,arg = parse_uri s in
-      match protocols#get proto with
-        | Some handler -> Some handler.static
-        | None -> None
+    match parse_uri s with
+      | Some (proto,arg) ->
+          begin match protocols#get proto with
+            | Some handler -> Some handler.static
+            | None -> None
+          end
+      | None -> None
 
 (** Resolving engine. *)
 
@@ -602,25 +558,34 @@ let resolve t timeout =
     (* If the file is local we only need to check that it's valid,
      * we'll actually do that in a single local_check for all local indicators
      * on the top of the stack. *)
-    if Sys.file_exists i.string then local_check t else
-    let proto,arg = parse_uri i.string in
-      match protocols#get proto with
-        | Some handler ->
-            add_log t
-              (Printf.sprintf
-                 "Resolving %S (timeout %.fs)..."
-                 i.string timeout) ;
-            let production = handler.resolve ~log:(add_log t) arg maxtime in
-              if production = [] then begin
-                log#f 4
-                  "Failed to resolve %S! \
-                   For more info, see server command 'trace %d'."
-                  i.string t.id ;
-                ignore (pop_indicator t)
-              end else
-                push_indicators t production
+    if file_exists i.string then local_check t else
+      match parse_uri i.string with
+        | Some (proto,arg) ->
+            begin match protocols#get proto with
+              | Some handler ->
+                  add_log t
+                    (Printf.sprintf
+                       "Resolving %S (timeout %.fs)..."
+                       i.string timeout) ;
+                  let production =
+                    handler.resolve ~log:(add_log t) arg maxtime
+                  in
+                    if production = [] then begin
+                      log#f 4
+                        "Failed to resolve %S! \
+                         For more info, see server command 'trace %d'."
+                        i.string t.id ;
+                      ignore (pop_indicator t)
+                    end else
+                      push_indicators t production
+              | None ->
+                  log#f 3 "Unknown protocol %S in URI %S!" proto i.string ;
+                  add_log t "Unknown protocol!" ;
+                  pop_indicator t
+            end
         | None ->
-            add_log t "Unknown protocol!" ;
+            log#f 3 "Nonexistent file or ill-formed URI %S!" i.string ;
+            add_log t "Nonexistent file or ill-formed URI!" ;
             pop_indicator t
   in
   let result =
